#!/usr/bin/env python

"""
Interaction with the mail system for the manager interface.

Copyright (C) 2014, 2015 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from imiptools.client import Client
from imiptools.data import get_address, get_uri, make_freebusy, \
                           to_part, uri_item, uri_items, uri_values
from imiptools.dates import format_datetime, get_timestamp, to_utc_datetime
from imiptools.handlers import Handler
from imiptools.period import update_freebusy
from imipweb.data import event_period_from_period

class ManagerHandler(Handler):

    """
    A content handler for use by the manager, as opposed to operating within the
    mail processing pipeline.
    """

    def __init__(self, obj, user, messenger):
        Handler.__init__(self, messenger=messenger)
        Client.__init__(self, user) # this redefines the Handler initialisation

        self.set_object(obj)

    # Communication methods.

    def send_message(self, method, sender, from_organiser, parts=None):

        """
        Create a full calendar object employing the given 'method', and send it
        to the appropriate recipients, also sending a copy to the 'sender'. The
        'from_organiser' value indicates whether the organiser is sending this
        message.
        """

        parts = parts or [self.obj.to_part(method)]

        # As organiser, send an invitation to attendees, excluding oneself if
        # also attending. The updated event will be saved by the outgoing
        # handler.

        organiser = get_uri(self.obj.get_value("ORGANIZER"))
        attendees = uri_values(self.obj.get_values("ATTENDEE"))

        if from_organiser:
            recipients = [get_address(attendee) for attendee in attendees if attendee != self.user]
        else:
            recipients = [get_address(organiser)]

        # Bundle free/busy information if appropriate.

        if self.is_sharing() and self.is_bundling():

            # Invent a unique identifier.

            utcnow = get_timestamp()
            uid = "imip-agent-%s-%s" % (utcnow, get_address(self.user))

            freebusy = self.store.get_freebusy(self.user)

            # Replace the non-updated free/busy details for this event with
            # newer details (since the outgoing handler updates this user's
            # free/busy details).

            update_freebusy(freebusy,
                self.obj.get_periods_for_freebusy(self.get_tzid(), self.get_window_end()),
                self.obj.get_value("TRANSP") or "OPAQUE",
                self.uid, self.recurrenceid,
                self.obj.get_value("SUMMARY"),
                organiser)

            user_attr = self.messenger and self.messenger.sender != get_address(self.user) and \
                {"SENT-BY" : get_uri(self.messenger.sender)} or {}

            parts.append(to_part("PUBLISH", [
                make_freebusy(freebusy, uid, self.user, user_attr)
                ]))

        # Explicitly specify the outgoing BCC recipient since we are sending as
        # the generic calendar user.

        message = self.messenger.make_outgoing_message(parts, recipients, outgoing_bcc=sender)
        self.messenger.sendmail(recipients, message.as_string(), outgoing_bcc=sender)

    # Action methods.

    def process_received_request(self):

        """
        Process the current request for the given 'user'. Return whether any
        action was taken.
        """

        # Reply only on behalf of this user.

        for attendee, attendee_attr in uri_items(self.obj.get_items("ATTENDEE")):

            if attendee == self.user:
                if attendee_attr.has_key("RSVP"):
                    del attendee_attr["RSVP"]
                if self.messenger and self.messenger.sender != get_address(attendee):
                    attendee_attr["SENT-BY"] = get_uri(self.messenger.sender)
                self.obj["ATTENDEE"] = [(attendee, attendee_attr)]

                self.update_dtstamp()
                self.set_sequence(False)

                self.send_message("REPLY", get_address(attendee), from_organiser=False)

                return True

        return False

    def process_created_request(self, method, to_cancel=None, to_unschedule=None):

        """
        Process the current request, sending a created request of the given
        'method' to attendees. Return whether any action was taken.

        If 'to_cancel' is specified, a list of participants to be sent cancel
        messages is provided.
        """

        organiser, organiser_attr = uri_item(self.obj.get_item("ORGANIZER"))

        if self.messenger and self.messenger.sender != get_address(organiser):
            organiser_attr["SENT-BY"] = get_uri(self.messenger.sender)

        self.update_dtstamp()
        self.set_sequence(True)

        parts = [self.obj.to_part(method)]

        # Add message parts with cancelled occurrence information.
        # NOTE: This could probably be merged with the updated event message.

        if to_unschedule:
            obj = self.obj.copy()
            obj.remove_all(["RRULE", "RDATE", "DTSTART", "DTEND", "DURATION"])

            for p in to_unschedule:
                if not p.origin:
                    continue
                date_tzid = self.get_tzid()
                obj["RECURRENCE-ID"] = [(format_datetime(to_utc_datetime(p.start, date_tzid)), {})]
                parts.append(obj.to_part("CANCEL"))

        # Send the updated event, along with a cancellation for each of the
        # unscheduled occurrences.

        self.send_message("CANCEL", get_address(organiser), from_organiser=True, parts=parts)

        # When cancelling, replace the attendees with those for whom the event
        # is now cancelled.

        if to_cancel:
            obj = self.obj.copy()
            obj["ATTENDEE"] = to_cancel

            # Send a cancellation to all uninvited attendees.

            self.send_message("CANCEL", get_address(organiser), from_organiser=True)

        return True

# vim: tabstop=4 expandtab shiftwidth=4
